@@ -4,13 +4,14 @@
 #pragma once
 
 #include <QObject>
+#include <QQmlEngine>
 #include <QQuickTextDocument>
 #include <QTextCursor>
 
+#include "chatbarcache.h"
 #include "models/completionmodel.h"
 #include "neochatroom.h"
 
-class QTextDocument;
 class NeoChatRoom;
 class SyntaxHighlighter;
 
@@ -59,14 +60,7 @@ class SyntaxHighlighter;
 class ChatDocumentHandler : public QObject
 {
     Q_OBJECT
-
-    /**
-     * @brief Is the instance being used to handle an edit message.
-     *
-     * This is needed to ensure that the text and mentions are saved and retrieved
-     * from the correct parameters in the assigned room.
-     */
-    Q_PROPERTY(bool isEdit READ isEdit WRITE setIsEdit NOTIFY isEditChanged)
+    QML_ELEMENT
 
     /**
      * @brief The QQuickTextDocument that is being handled.
@@ -94,29 +88,31 @@ class ChatDocumentHandler : public QObject
      * This is typically provided to a qml component to visualise the current
      * completion results.
      */
-    Q_PROPERTY(CompletionModel *completionModel READ completionModel NOTIFY completionModelChanged)
+    Q_PROPERTY(CompletionModel *completionModel READ completionModel CONSTANT)
 
     /**
      * @brief The current room that the the text document is being handled for.
      */
     Q_PROPERTY(NeoChatRoom *room READ room WRITE setRoom NOTIFY roomChanged)
 
+    /**
+     * @brief The cache for the chat bar the text document is being handled for.
+     */
+    Q_PROPERTY(ChatBarCache *chatBarCache READ chatBarCache WRITE setChatBarCache NOTIFY chatBarCacheChanged)
+
     /**
      * @brief The color to highlight user mentions.
      */
-    Q_PROPERTY(QColor mentionColor READ mentionColor WRITE setMentionColor NOTIFY mentionColorChanged);
+    Q_PROPERTY(QColor mentionColor READ mentionColor WRITE setMentionColor NOTIFY mentionColorChanged)
 
     /**
      * @brief The color to highlight spelling errors.
      */
-    Q_PROPERTY(QColor errorColor READ errorColor WRITE setErrorColor NOTIFY errorColorChanged);
+    Q_PROPERTY(QColor errorColor READ errorColor WRITE setErrorColor NOTIFY errorColorChanged)
 
 public:
     explicit ChatDocumentHandler(QObject *parent = nullptr);
 
-    [[nodiscard]] bool isEdit() const;
-    void setIsEdit(bool edit);
-
     [[nodiscard]] QQuickTextDocument *document() const;
     void setDocument(QQuickTextDocument *document);
 
@@ -132,9 +128,11 @@ public:
     [[nodiscard]] NeoChatRoom *room() const;
     void setRoom(NeoChatRoom *room);
 
+    [[nodiscard]] ChatBarCache *chatBarCache() const;
+    void setChatBarCache(ChatBarCache *chatBarCache);
+
     Q_INVOKABLE void complete(int index);
 
-    void updateCompletions();
     CompletionModel *completionModel() const;
 
     [[nodiscard]] QColor mentionColor() const;
@@ -144,11 +142,10 @@ public:
     void setErrorColor(const QColor &color);
 
 Q_SIGNALS:
-    void isEditChanged();
     void documentChanged();
     void cursorPositionChanged();
     void roomChanged();
-    void completionModelChanged();
+    void chatBarCacheChanged();
     void selectionStartChanged();
     void selectionEndChanged();
     void errorColorChanged();
@@ -157,11 +154,10 @@ Q_SIGNALS:
 private:
     int completionStartIndex() const;
 
-    bool m_isEdit = false;
-
     QPointer<QQuickTextDocument> m_document;
 
     QPointer<NeoChatRoom> m_room;
+    QPointer<ChatBarCache> m_chatBarCache;
     bool completionVisible = false;
 
     QColor m_mentionColor;
