@@ -3,6 +3,7 @@
 
 #include "actionsmodel.h"
 
+#include "chatbarcache.h"
 #include "controller.h"
 #include "neochatroom.h"
 #include "roommanager.h"
@@ -14,11 +15,12 @@
 using Action = ActionsModel::Action;
 using namespace Quotient;
 
-QStringList rainbowColors{"#ff2b00", "#ff5500", "#ff8000", "#ffaa00", "#ffd500", "#ffff00", "#d4ff00", "#aaff00", "#80ff00", "#55ff00", "#2bff00", "#00ff00",
-                          "#00ff2b", "#00ff55", "#00ff80", "#00ffaa", "#00ffd5", "#00ffff", "#00d4ff", "#00aaff", "#007fff", "#0055ff", "#002bff", "#0000ff",
-                          "#2a00ff", "#5500ff", "#7f00ff", "#aa00ff", "#d400ff", "#ff00ff", "#ff00d4", "#ff00aa", "#ff0080", "#ff0055", "#ff002b", "#ff0000"};
+QStringList rainbowColors{"#ff2b00"_ls, "#ff5500"_ls, "#ff8000"_ls, "#ffaa00"_ls, "#ffd500"_ls, "#ffff00"_ls, "#d4ff00"_ls, "#aaff00"_ls, "#80ff00"_ls,
+                          "#55ff00"_ls, "#2bff00"_ls, "#00ff00"_ls, "#00ff2b"_ls, "#00ff55"_ls, "#00ff80"_ls, "#00ffaa"_ls, "#00ffd5"_ls, "#00ffff"_ls,
+                          "#00d4ff"_ls, "#00aaff"_ls, "#007fff"_ls, "#0055ff"_ls, "#002bff"_ls, "#0000ff"_ls, "#2a00ff"_ls, "#5500ff"_ls, "#7f00ff"_ls,
+                          "#aa00ff"_ls, "#d400ff"_ls, "#ff00ff"_ls, "#ff00d4"_ls, "#ff00aa"_ls, "#ff0080"_ls, "#ff0055"_ls, "#ff002b"_ls, "#ff0000"_ls};
 
-auto leaveRoomLambda = [](const QString &text, NeoChatRoom *room) {
+auto leaveRoomLambda = [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
     if (text.isEmpty()) {
         Q_EMIT room->showMessage(NeoChatRoom::Info, i18n("Leaving this room."));
         room->connection()->leaveRoom(room);
@@ -44,7 +46,7 @@ auto leaveRoomLambda = [](const QString &text, NeoChatRoom *room) {
     return QString();
 };
 
-auto roomNickLambda = [](const QString &text, NeoChatRoom *room) {
+auto roomNickLambda = [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
     if (text.isEmpty()) {
         Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("No new nickname provided, no changes will happen."));
     } else {
@@ -53,10 +55,10 @@ auto roomNickLambda = [](const QString &text, NeoChatRoom *room) {
     return QString();
 };
 
-QVector<ActionsModel::Action> actions{
+QList<ActionsModel::Action> actions{
     Action{
         QStringLiteral("shrug"),
-        [](const QString &message, NeoChatRoom *) {
+        [](const QString &message, NeoChatRoom *, ChatBarCache *) {
             return QStringLiteral("¯\\\\_(ツ)_/¯ %1").arg(message);
         },
         true,
@@ -66,7 +68,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("lenny"),
-        [](const QString &message, NeoChatRoom *) {
+        [](const QString &message, NeoChatRoom *, ChatBarCache *) {
             return QStringLiteral("( ͡° ͜ʖ ͡°) %1").arg(message);
         },
         true,
@@ -76,7 +78,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("tableflip"),
-        [](const QString &message, NeoChatRoom *) {
+        [](const QString &message, NeoChatRoom *, ChatBarCache *) {
             return QStringLiteral("(╯°□°）╯︵ ┻━┻ %1").arg(message);
         },
         true,
@@ -86,7 +88,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("unflip"),
-        [](const QString &message, NeoChatRoom *) {
+        [](const QString &message, NeoChatRoom *, ChatBarCache *) {
             return QStringLiteral("┬──┬ ノ( ゜-゜ノ) %1").arg(message);
         },
         true,
@@ -96,7 +98,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("rainbow"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *chatBarCache) {
             QString rainbowText;
             for (int i = 0; i < text.length(); i++) {
                 rainbowText += QStringLiteral("<font color='%2'>%3</font>").arg(rainbowColors[i % rainbowColors.length()], text.at(i));
@@ -105,8 +107,8 @@ QVector<ActionsModel::Action> actions{
             room->postMessage(QStringLiteral("/rainbow %1").arg(text),
                               rainbowText,
                               RoomMessageEvent::MsgType::Text,
-                              room->chatBoxReplyId(),
-                              room->chatBoxEditId());
+                              chatBarCache->replyId(),
+                              chatBarCache->editId());
             return QString();
         },
         false,
@@ -116,7 +118,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("rainbowme"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *chatBarCache) {
             QString rainbowText;
             for (int i = 0; i < text.length(); i++) {
                 rainbowText += QStringLiteral("<font color='%2'>%3</font>").arg(rainbowColors[i % rainbowColors.length()], text.at(i));
@@ -125,8 +127,8 @@ QVector<ActionsModel::Action> actions{
             room->postMessage(QStringLiteral("/rainbow %1").arg(text),
                               rainbowText,
                               RoomMessageEvent::MsgType::Emote,
-                              room->chatBoxReplyId(),
-                              room->chatBoxEditId());
+                              chatBarCache->replyId(),
+                              chatBarCache->editId());
             return QString();
         },
         false,
@@ -136,7 +138,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("plain"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             room->postMessage(text, text.toHtmlEscaped(), RoomMessageEvent::MsgType::Text, {}, {});
             return QString();
         },
@@ -147,13 +149,13 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("spoiler"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *chatBarCache) {
             // Ideally, we would just return rainbowText and let that do the rest, but the colors don't survive markdownToHTML.
             room->postMessage(QStringLiteral("/spoiler %1").arg(text),
                               QStringLiteral("<span data-mx-spoiler>%1</span>").arg(text),
                               RoomMessageEvent::MsgType::Text,
-                              room->chatBoxReplyId(),
-                              room->chatBoxEditId());
+                              chatBarCache->replyId(),
+                              chatBarCache->editId());
             return QString();
         },
         false,
@@ -163,7 +165,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("me"),
-        [](const QString &text, NeoChatRoom *) {
+        [](const QString &text, NeoChatRoom *, ChatBarCache *) {
             return text;
         },
         true,
@@ -173,7 +175,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("notice"),
-        [](const QString &text, NeoChatRoom *) {
+        [](const QString &text, NeoChatRoom *, ChatBarCache *) {
             return text;
         },
         true,
@@ -183,7 +185,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("invite"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             static const QRegularExpression mxidRegex(
                 QStringLiteral(R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"));
             auto regexMatch = mxidRegex.match(text);
@@ -219,7 +221,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("join"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             QRegularExpression roomRegex(QStringLiteral(R"(^[#!][^:]+:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?)"));
             auto regexMatch = roomRegex.match(text);
             if (!regexMatch.hasMatch()) {
@@ -243,7 +245,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("knock"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             auto parts = text.split(QLatin1String(" "));
             QString roomName = parts[0];
             QRegularExpression roomRegex(QStringLiteral(R"(^[#!][^:]+:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?)"));
@@ -259,8 +261,8 @@ QVector<ActionsModel::Action> actions{
                 return QString();
             }
             Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Knocking room <roomname>.", "Knocking room %1.", text));
-            auto connection = Controller::instance().activeConnection();
-            const auto knownServer = roomName.mid(roomName.indexOf(":") + 1);
+            auto connection = room->connection();
+            const auto knownServer = roomName.mid(roomName.indexOf(":"_ls) + 1);
             if (parts.length() >= 2) {
                 RoomManager::instance().knockRoom(connection, roomName, parts[1], QStringList{knownServer});
             } else {
@@ -275,7 +277,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("j"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             QRegularExpression roomRegex(QStringLiteral(R"(^[#!][^:]+:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?)"));
             auto regexMatch = roomRegex.match(text);
             if (!regexMatch.hasMatch()) {
@@ -283,7 +285,7 @@ QVector<ActionsModel::Action> actions{
                                          i18nc("'<text>' does not look like a room id or alias.", "'%1' does not look like a room id or alias.", text));
                 return QString();
             }
-            if (Controller::instance().activeConnection()->room(text) || Controller::instance().activeConnection()->roomByAlias(text)) {
+            if (room->connection()->room(text) || room->connection()->roomByAlias(text)) {
                 Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("You are already in room <roomname>.", "You are already in room %1.", text));
                 return QString();
             }
@@ -314,7 +316,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("nick"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             if (text.isEmpty()) {
                 Q_EMIT room->showMessage(NeoChatRoom::Error, i18n("No new nickname provided, no changes will happen."));
             } else {
@@ -345,7 +347,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("ignore"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             static const QRegularExpression mxidRegex(
                 QStringLiteral(R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"));
             auto regexMatch = mxidRegex.match(text);
@@ -373,7 +375,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("unignore"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             static const QRegularExpression mxidRegex(
                 QStringLiteral(R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"));
             auto regexMatch = mxidRegex.match(text);
@@ -401,9 +403,8 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("react"),
-        [](const QString &text, NeoChatRoom *room) {
-            QString replyEventId = room->chatBoxReplyId();
-            if (replyEventId.isEmpty()) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *chatBarCache) {
+            if (chatBarCache->replyId().isEmpty()) {
                 for (auto it = room->messageEvents().crbegin(); it != room->messageEvents().crend(); it++) {
                     const auto &evt = **it;
                     if (const auto event = eventCast<const RoomMessageEvent>(&evt)) {
@@ -412,7 +413,7 @@ QVector<ActionsModel::Action> actions{
                     }
                 }
             }
-            room->toggleReaction(replyEventId, text);
+            room->toggleReaction(chatBarCache->replyId(), text);
             return QString();
         },
         false,
@@ -422,7 +423,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("ban"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             auto parts = text.split(QLatin1String(" "));
             static const QRegularExpression mxidRegex(
                 QStringLiteral(R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"));
@@ -450,7 +451,7 @@ QVector<ActionsModel::Action> actions{
                     i18nc("You are not allowed to ban <username> from this room.", "You are not allowed to ban %1 from this room.", parts[0]));
                 return QString();
             }
-            room->ban(parts[0], parts.size() > 1 ? parts.mid(1).join(" ") : QString());
+            room->ban(parts[0], parts.size() > 1 ? parts.mid(1).join(QLatin1Char(' ')) : QString());
             Q_EMIT room->showMessage(NeoChatRoom::Positive, i18nc("<username> was banned from this room.", "%1 was banned from this room.", parts[0]));
             return QString();
         },
@@ -461,7 +462,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("unban"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             static const QRegularExpression mxidRegex(
                 QStringLiteral(R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"));
             auto regexMatch = mxidRegex.match(text);
@@ -494,7 +495,7 @@ QVector<ActionsModel::Action> actions{
     },
     Action{
         QStringLiteral("kick"),
-        [](const QString &text, NeoChatRoom *room) {
+        [](const QString &text, NeoChatRoom *room, ChatBarCache *) {
             auto parts = text.split(QLatin1String(" "));
             static const QRegularExpression mxidRegex(
                 QStringLiteral(R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"));
@@ -527,7 +528,7 @@ QVector<ActionsModel::Action> actions{
                     i18nc("You are not allowed to kick <username> from this room", "You are not allowed to kick %1 from this room.", parts[0]));
                 return QString();
             }
-            room->kickMember(parts[0], parts.size() > 1 ? parts.mid(1).join(" ") : QString());
+            room->kickMember(parts[0], parts.size() > 1 ? parts.mid(1).join(QLatin1Char(' ')) : QString());
             Q_EMIT room->showMessage(NeoChatRoom::Positive, i18nc("<username> was kicked from this room.", "%1 was kicked from this room.", parts[0]));
             return QString();
         },
@@ -573,7 +574,7 @@ QHash<int, QByteArray> ActionsModel::roleNames() const
     };
 }
 
-QVector<Action> &ActionsModel::allActions() const
+QList<Action> &ActionsModel::allActions() const
 {
     return actions;
 }
