@@ -66,8 +66,8 @@ QVariant CompletionModel::data(const QModelIndex &index, int role) const
 
     if (m_autoCompletionType == Command) {
         if (role == DisplayNameRole) {
-            return m_filterModel->data(filterIndex, ActionsModel::Prefix).toString() + QStringLiteral(" ")
-                + m_filterModel->data(filterIndex, ActionsModel::Parameters).toString();
+            return QStringLiteral("%1 %2").arg(m_filterModel->data(filterIndex, ActionsModel::Prefix).toString(),
+                                               m_filterModel->data(filterIndex, ActionsModel::Parameters).toString());
         }
         if (role == SubtitleRole) {
             return m_filterModel->data(filterIndex, ActionsModel::Description);
@@ -87,7 +87,11 @@ QVariant CompletionModel::data(const QModelIndex &index, int role) const
             return m_filterModel->data(filterIndex, RoomListModel::CanonicalAliasRole);
         }
         if (role == IconNameRole) {
-            return m_filterModel->data(filterIndex, RoomListModel::AvatarRole);
+            auto mediaId = m_filterModel->data(filterIndex, RoomListModel::AvatarRole).toString();
+            if (mediaId.isEmpty()) {
+                return QVariant();
+            }
+            return m_room->connection()->makeMediaUrl(QUrl(QStringLiteral("mxc://%1").arg(mediaId)));
         }
     }
     if (m_autoCompletionType == Emoji) {
@@ -144,7 +148,7 @@ void CompletionModel::updateCompletion()
         m_filterModel->setFullText(m_fullText);
         m_filterModel->setFilterText(m_text);
         m_filterModel->invalidate();
-    } else if (text().startsWith(QLatin1Char(':')) && !text()[1].isUpper()
+    } else if (text().startsWith(QLatin1Char(':')) && text().size() > 1 && !text()[1].isUpper()
                && (m_fullText.indexOf(QLatin1Char(':'), 1) == -1
                    || (m_fullText.indexOf(QLatin1Char(' ')) != -1 && m_fullText.indexOf(QLatin1Char(':'), 1) > m_fullText.indexOf(QLatin1Char(' '), 1)))) {
         m_filterModel->setSourceModel(m_emojiModel);
