@@ -15,20 +15,35 @@
 
 using namespace Quotient;
 
+void CustomEmojiModel::setConnection(NeoChatConnection *connection)
+{
+    if (connection == m_connection) {
+        return;
+    }
+    m_connection = connection;
+    Q_EMIT connectionChanged();
+    fetchEmojis();
+}
+
+NeoChatConnection *CustomEmojiModel::connection() const
+{
+    return m_connection;
+}
+
 void CustomEmojiModel::fetchEmojis()
 {
-    if (!Controller::instance().activeConnection()) {
+    if (!m_connection) {
         return;
     }
 
-    const auto &data = Controller::instance().activeConnection()->accountData("im.ponies.user_emotes");
+    const auto &data = m_connection->accountData("im.ponies.user_emotes"_ls);
     if (data == nullptr) {
         return;
     }
-    QJsonObject emojis = data->contentJson()["images"].toObject();
+    QJsonObject emojis = data->contentJson()["images"_ls].toObject();
 
     // TODO: Remove with stable migration
-    const auto legacyEmojis = data->contentJson()["emoticons"].toObject();
+    const auto legacyEmojis = data->contentJson()["emoticons"_ls].toObject();
     for (const auto &emoji : legacyEmojis.keys()) {
         if (!emojis.contains(emoji)) {
             emojis[emoji] = legacyEmojis[emoji];
@@ -41,9 +56,9 @@ void CustomEmojiModel::fetchEmojis()
     for (const auto &emoji : emojis.keys()) {
         const auto &data = emojis[emoji];
 
-        const auto e = emoji.startsWith(":") ? emoji : (QStringLiteral(":") + emoji + QStringLiteral(":"));
+        const auto e = emoji.startsWith(":"_ls) ? emoji : (QStringLiteral(":") + emoji + QStringLiteral(":"));
 
-        m_emojis << CustomEmoji{e, data.toObject()["url"].toString(), QRegularExpression(e)};
+        m_emojis << CustomEmoji{e, data.toObject()["url"_ls].toString(), QRegularExpression(e)};
     }
 
     endResetModel();
@@ -53,22 +68,22 @@ void CustomEmojiModel::addEmoji(const QString &name, const QUrl &location)
 {
     using namespace Quotient;
 
-    auto job = Controller::instance().activeConnection()->uploadFile(location.toLocalFile());
+    auto job = m_connection->uploadFile(location.toLocalFile());
 
-    connect(job, &BaseJob::success, this, [name, location, job] {
-        const auto &data = Controller::instance().activeConnection()->accountData("im.ponies.user_emotes");
+    connect(job, &BaseJob::success, this, [name, location, job, this] {
+        const auto &data = m_connection->accountData("im.ponies.user_emotes"_ls);
         auto json = data != nullptr ? data->contentJson() : QJsonObject();
-        auto emojiData = json["images"].toObject();
+        auto emojiData = json["images"_ls].toObject();
 
         QString url;
         url = job->contentUri().toString();
 
         QImage image(location.toLocalFile());
         QJsonObject imageInfo;
-        imageInfo["w"] = image.width();
-        imageInfo["h"] = image.height();
-        imageInfo["mimetype"] = QMimeDatabase().mimeTypeForFile(location.toLocalFile()).name();
-        imageInfo["size"] = image.sizeInBytes();
+        imageInfo["w"_ls] = image.width();
+        imageInfo["h"_ls] = image.height();
+        imageInfo["mimetype"_ls] = QMimeDatabase().mimeTypeForFile(location.toLocalFile()).name();
+        imageInfo["size"_ls] = image.sizeInBytes();
 
         emojiData[QStringLiteral("%1").arg(name)] = QJsonObject({
             {QStringLiteral("url"), url},
@@ -77,8 +92,8 @@ void CustomEmojiModel::addEmoji(const QString &name, const QUrl &location)
             {"usage"_ls, "emoticon"_ls},
         });
 
-        json["images"] = emojiData;
-        Controller::instance().activeConnection()->setAccountData("im.ponies.user_emotes", json);
+        json["images"_ls] = emojiData;
+        m_connection->setAccountData("im.ponies.user_emotes"_ls, json);
     });
 }
 
@@ -86,41 +101,41 @@ void CustomEmojiModel::removeEmoji(const QString &name)
 {
     using namespace Quotient;
 
-    const auto &data = Controller::instance().activeConnection()->accountData("im.ponies.user_emotes");
+    const auto &data = m_connection->accountData("im.ponies.user_emotes"_ls);
     Q_ASSERT(data);
     auto json = data->contentJson();
     const QString _name = name.mid(1).chopped(1);
-    auto emojiData = json["images"].toObject();
+    auto emojiData = json["images"_ls].toObject();
 
     if (emojiData.contains(name)) {
         emojiData.remove(name);
-        json["images"] = emojiData;
+        json["images"_ls] = emojiData;
     }
     if (emojiData.contains(_name)) {
         emojiData.remove(_name);
-        json["images"] = emojiData;
+        json["images"_ls] = emojiData;
     }
-    emojiData = json["emoticons"].toObject();
+    emojiData = json["emoticons"_ls].toObject();
     if (emojiData.contains(name)) {
         emojiData.remove(name);
-        json["emoticons"] = emojiData;
+        json["emoticons"_ls] = emojiData;
     }
     if (emojiData.contains(_name)) {
         emojiData.remove(_name);
-        json["emoticons"] = emojiData;
+        json["emoticons"_ls] = emojiData;
     }
-    Controller::instance().activeConnection()->setAccountData("im.ponies.user_emotes", json);
+    m_connection->setAccountData("im.ponies.user_emotes"_ls, json);
 }
 
 CustomEmojiModel::CustomEmojiModel(QObject *parent)
     : QAbstractListModel(parent)
 {
-    connect(&Controller::instance(), &Controller::activeConnectionChanged, this, [this]() {
-        if (!Controller::instance().activeConnection()) {
+    connect(this, &CustomEmojiModel::connectionChanged, this, [this]() {
+        if (!m_connection) {
             return;
         }
         CustomEmojiModel::fetchEmojis();
-        connect(Controller::instance().activeConnection(), &Connection::accountDataChanged, this, [this](const QString &id) {
+        connect(m_connection, &Connection::accountDataChanged, this, [this](const QString &id) {
             if (id != QStringLiteral("im.ponies.user_emotes")) {
                 return;
             }
@@ -174,7 +189,7 @@ QHash<int, QByteArray> CustomEmojiModel::roleNames() const
 
 QString CustomEmojiModel::preprocessText(const QString &text)
 {
-    auto parts = text.split("```");
+    auto parts = text.split("```"_ls);
     bool skip = true;
     for (auto &part : parts) {
         skip = !skip;
@@ -187,7 +202,7 @@ QString CustomEmojiModel::preprocessText(const QString &text)
                 QStringLiteral(R"(<img data-mx-emoticon="" src="%1" alt="%2" title="%2" height="32" vertical-align="middle" />)").arg(emoji.url, emoji.name));
         }
     }
-    return parts.join("```");
+    return parts.join("```"_ls);
 }
 
 QVariantList CustomEmojiModel::filterModel(const QString &filter)
