@@ -4,9 +4,12 @@
 #pragma once
 
 #include <QAbstractListModel>
+#include <QQmlEngine>
 #include <QRegularExpression>
 #include <memory>
 
+class NeoChatConnection;
+
 struct CustomEmoji {
     QString name; // with :semicolons:
     QString url; // mxc://
@@ -27,6 +30,10 @@ struct CustomEmoji {
 class CustomEmojiModel : public QAbstractListModel
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+
+    Q_PROPERTY(NeoChatConnection *connection READ connection WRITE setConnection NOTIFY connectionChanged)
 
 public:
     /**
@@ -48,6 +55,11 @@ public:
         static CustomEmojiModel _instance;
         return _instance;
     }
+    static CustomEmojiModel *create(QQmlEngine *engine, QJSEngine *)
+    {
+        engine->setObjectOwnership(&instance(), QQmlEngine::CppOwnership);
+        return &instance();
+    }
 
     /**
      * @brief Get the given role value at the given index.
@@ -90,9 +102,16 @@ public:
      */
     Q_INVOKABLE void removeEmoji(const QString &name);
 
+    void setConnection(NeoChatConnection *connection);
+    NeoChatConnection *connection() const;
+
+Q_SIGNALS:
+    void connectionChanged();
+
 private:
     explicit CustomEmojiModel(QObject *parent = nullptr);
     QList<CustomEmoji> m_emojis;
+    NeoChatConnection *m_connection = nullptr;
 
     void fetchEmojis();
 };
