@@ -25,7 +25,7 @@ void PublicRoomListModel::setConnection(Connection *conn)
 
     beginResetModel();
 
-    nextBatch = "";
+    nextBatch = QString();
     attempted = false;
     rooms.clear();
     m_server.clear();
@@ -41,6 +41,7 @@ void PublicRoomListModel::setConnection(Connection *conn)
     if (job) {
         job->abandon();
         job = nullptr;
+        Q_EMIT loadingChanged();
     }
 
     if (m_connection) {
@@ -67,15 +68,17 @@ void PublicRoomListModel::setServer(const QString &value)
 
     beginResetModel();
 
-    nextBatch = "";
+    nextBatch = QString();
     attempted = false;
     rooms.clear();
+    Q_EMIT loadingChanged();
 
     endResetModel();
 
     if (job) {
         job->abandon();
         job = nullptr;
+        Q_EMIT loadingChanged();
     }
 
     if (m_connection) {
@@ -101,7 +104,7 @@ void PublicRoomListModel::setKeyword(const QString &value)
 
     beginResetModel();
 
-    nextBatch = "";
+    nextBatch = QString();
     attempted = false;
     rooms.clear();
 
@@ -110,6 +113,7 @@ void PublicRoomListModel::setKeyword(const QString &value)
     if (job) {
         job->abandon();
         job = nullptr;
+        Q_EMIT loadingChanged();
     }
 
     if (m_connection) {
@@ -120,6 +124,20 @@ void PublicRoomListModel::setKeyword(const QString &value)
     Q_EMIT hasMoreChanged();
 }
 
+bool PublicRoomListModel::showOnlySpaces() const
+{
+    return m_showOnlySpaces;
+}
+
+void PublicRoomListModel::setShowOnlySpaces(bool showOnlySpaces)
+{
+    if (showOnlySpaces == m_showOnlySpaces) {
+        return;
+    }
+    m_showOnlySpaces = showOnlySpaces;
+    Q_EMIT showOnlySpacesChanged();
+}
+
 void PublicRoomListModel::next(int count)
 {
     if (count < 1) {
@@ -132,7 +150,12 @@ void PublicRoomListModel::next(int count)
         return;
     }
 
-    job = m_connection->callApi<QueryPublicRoomsJob>(m_server, count, nextBatch, QueryPublicRoomsJob::Filter{m_keyword, {}});
+    QStringList roomTypes;
+    if (m_showOnlySpaces) {
+        roomTypes += QLatin1String("m.space");
+    }
+    job = m_connection->callApi<QueryPublicRoomsJob>(m_server, count, nextBatch, QueryPublicRoomsJob::Filter{m_keyword, roomTypes});
+    Q_EMIT loadingChanged();
 
     connect(job, &BaseJob::finished, this, [this] {
         attempted = true;
@@ -150,6 +173,7 @@ void PublicRoomListModel::next(int count)
         }
 
         this->job = nullptr;
+        Q_EMIT loadingChanged();
     });
 }
 
@@ -165,7 +189,7 @@ QVariant PublicRoomListModel::data(const QModelIndex &index, int role) const
         return {};
     }
     auto room = rooms.at(index.row());
-    if (role == NameRole) {
+    if (role == DisplayNameRole) {
         auto displayName = room.name;
         if (!displayName.isEmpty()) {
             return displayName;
@@ -182,18 +206,17 @@ QVariant PublicRoomListModel::data(const QModelIndex &index, int role) const
 
         return room.roomId;
     }
-    if (role == AvatarRole) {
+    if (role == AvatarUrlRole) {
         auto avatarUrl = room.avatarUrl;
-
-        if (avatarUrl.isEmpty()) {
-            return "";
+        if (avatarUrl.isEmpty() || !m_connection) {
+            return QUrl();
         }
-        return avatarUrl.url().remove(0, 6);
+        return m_connection->makeMediaUrl(avatarUrl);
     }
     if (role == TopicRole) {
         return room.topic;
     }
-    if (role == RoomIDRole) {
+    if (role == RoomIdRole) {
         return room.roomId;
     }
     if (role == AliasRole) {
@@ -226,10 +249,10 @@ QHash<int, QByteArray> PublicRoomListModel::roleNames() const
 {
     QHash<int, QByteArray> roles;
 
-    roles[NameRole] = "name";
-    roles[AvatarRole] = "avatar";
+    roles[DisplayNameRole] = "displayName";
+    roles[AvatarUrlRole] = "avatarUrl";
     roles[TopicRole] = "topic";
-    roles[RoomIDRole] = "roomID";
+    roles[RoomIdRole] = "roomId";
     roles[MemberCountRole] = "memberCount";
     roles[AllowGuestsRole] = "allowGuests";
     roles[WorldReadableRole] = "worldReadable";
@@ -253,4 +276,9 @@ bool PublicRoomListModel::hasMore() const
     return !(attempted && nextBatch.isEmpty());
 }
 
+bool PublicRoomListModel::loading() const
+{
+    return job != nullptr;
+}
+
 #include "moc_publicroomlistmodel.cpp"
