@@ -23,6 +23,9 @@ static const QHash<QString, QString> defaultRuleNames = {
     {QStringLiteral(".m.rule.encrypted"), QStringLiteral("Messages in encrypted group chats")},
     {QStringLiteral(".m.rule.tombstone"), QStringLiteral("Room upgrade messages")},
     {QStringLiteral(".m.rule.contains_display_name"), QStringLiteral("Messages containing my display name")},
+    {QStringLiteral(".m.rule.is_user_mention"), QStringLiteral("Messages which mention my Matrix user ID.")},
+    {QStringLiteral(".m.rule.is_room_mention"), QStringLiteral("Messages which mention a room.")},
+    {QStringLiteral(".m.rule.contains_user_name"), QStringLiteral("Messages containing the local part of my Matrix ID.")},
     {QStringLiteral(".m.rule.roomnotif"), QStringLiteral("Whole room (@room) notifications")},
     {QStringLiteral(".m.rule.invite_for_me"), QStringLiteral("Invites to a room")},
     {QStringLiteral(".m.rule.call"), QStringLiteral("Call invitation")},
@@ -37,6 +40,9 @@ static const QHash<QString, PushNotificationSection::Section> defaultSections =
     {QStringLiteral(".m.rule.encrypted"), PushNotificationSection::Room},
     {QStringLiteral(".m.rule.tombstone"), PushNotificationSection::Room},
     {QStringLiteral(".m.rule.contains_display_name"), PushNotificationSection::Mentions},
+    {QStringLiteral(".m.rule.is_user_mention"), PushNotificationSection::Mentions},
+    {QStringLiteral(".m.rule.is_room_mention"), PushNotificationSection::Mentions},
+    {QStringLiteral(".m.rule.contains_user_name"), PushNotificationSection::Mentions},
     {QStringLiteral(".m.rule.roomnotif"), PushNotificationSection::Mentions},
     {QStringLiteral(".m.rule.invite_for_me"), PushNotificationSection::Invites},
     {QStringLiteral(".m.rule.call"), PushNotificationSection::Undefined}, // TODO: make invites when VOIP added.
@@ -60,27 +66,16 @@ PushRuleModel::PushRuleModel(QObject *parent)
     : QAbstractListModel(parent)
 {
     m_defaultKeywordAction = static_cast<PushNotificationAction::Action>(NeoChatConfig::self()->keywordPushRuleDefault());
-
-    if (Controller::instance().activeConnection()) {
-        controllerConnectionChanged();
-    }
-    connect(&Controller::instance(), &Controller::activeConnectionChanged, this, &PushRuleModel::controllerConnectionChanged);
-}
-
-void PushRuleModel::controllerConnectionChanged()
-{
-    connect(Controller::instance().activeConnection(), &Quotient::Connection::accountDataChanged, this, &PushRuleModel::updateNotificationRules);
-    updateNotificationRules("m.push_rules");
 }
 
 void PushRuleModel::updateNotificationRules(const QString &type)
 {
-    if (type != "m.push_rules") {
+    if (type != QStringLiteral("m.push_rules")) {
         return;
     }
 
-    const QJsonObject ruleDataJson = Controller::instance().activeConnection()->accountDataJson("m.push_rules");
-    const Quotient::PushRuleset ruleData = Quotient::fromJson<Quotient::PushRuleset>(ruleDataJson["global"].toObject());
+    const QJsonObject ruleDataJson = m_connection->accountDataJson(QStringLiteral("m.push_rules"));
+    const Quotient::PushRuleset ruleData = Quotient::fromJson<Quotient::PushRuleset>(ruleDataJson[QStringLiteral("global")].toObject());
 
     beginResetModel();
     m_rules.clear();
@@ -98,7 +93,7 @@ void PushRuleModel::updateNotificationRules(const QString &type)
     endResetModel();
 }
 
-void PushRuleModel::setRules(QVector<Quotient::PushRule> rules, PushNotificationKind::Kind kind)
+void PushRuleModel::setRules(QList<Quotient::PushRule> rules, PushNotificationKind::Kind kind)
 {
     for (const auto &rule : rules) {
         QString roomId;
@@ -138,6 +133,9 @@ PushNotificationSection::Section PushRuleModel::getSection(Quotient::PushRule ru
     if (defaultSections.contains(ruleId)) {
         return defaultSections.value(ruleId);
     } else {
+        if (rule.ruleId.startsWith(u'.')) {
+            return PushNotificationSection::Unknown;
+        }
         /**
          * If the rule name resolves to a matrix id for a room that the user is part
          * of it shouldn't appear in the global list as it's overriding the global
@@ -145,8 +143,7 @@ PushNotificationSection::Section PushRuleModel::getSection(Quotient::PushRule ru
          *
          * Rooms that the user hasn't joined shouldn't have a rule.
          */
-        auto connection = Controller::instance().activeConnection();
-        if (connection->room(ruleId) != nullptr) {
+        if (m_connection->room(ruleId) != nullptr) {
             return PushNotificationSection::Undefined;
         }
         /**
@@ -160,7 +157,7 @@ PushNotificationSection::Section PushRuleModel::getSection(Quotient::PushRule ru
         if (!testUserId.startsWith(u'@')) {
             testUserId.prepend(u'@');
         }
-        if (connection->user(testUserId) != nullptr) {
+        if (testUserId.startsWith(u'@') && !Quotient::serverPart(testUserId).isEmpty() && m_connection->user(testUserId) != nullptr) {
             return PushNotificationSection::Undefined;
         }
         // If the rule has push conditions and one is a room ID it is a room only keyword.
@@ -201,7 +198,7 @@ bool PushRuleModel::globalNotificationsEnabled() const
 
 void PushRuleModel::setGlobalNotificationsEnabled(bool enabled)
 {
-    setNotificationRuleEnabled("override", ".m.rule.master", !enabled);
+    setNotificationRuleEnabled(QStringLiteral("override"), QStringLiteral(".m.rule.master"), !enabled);
 }
 
 bool PushRuleModel::globalNotificationsSet() const
@@ -296,32 +293,32 @@ void PushRuleModel::setPushRuleAction(const QString &id, PushNotificationAction:
 void PushRuleModel::addKeyword(const QString &keyword, const QString &roomId)
 {
     PushNotificationKind::Kind kind = PushNotificationKind::Content;
-    const QVector<QVariant> actions = actionToVariant(m_defaultKeywordAction);
-    QVector<Quotient::PushCondition> pushConditions;
+    const QList<QVariant> actions = actionToVariant(m_defaultKeywordAction);
+    QList<Quotient::PushCondition> pushConditions;
     if (!roomId.isEmpty()) {
         kind = PushNotificationKind::Override;
 
         Quotient::PushCondition roomCondition;
-        roomCondition.kind = "event_match";
-        roomCondition.key = "room_id";
+        roomCondition.kind = QStringLiteral("event_match");
+        roomCondition.key = QStringLiteral("room_id");
         roomCondition.pattern = roomId;
         pushConditions.append(roomCondition);
 
         Quotient::PushCondition keywordCondition;
-        keywordCondition.kind = "event_match";
-        keywordCondition.key = "content.body";
+        keywordCondition.kind = QStringLiteral("event_match");
+        keywordCondition.key = QStringLiteral("content.body");
         keywordCondition.pattern = keyword;
         pushConditions.append(keywordCondition);
     }
 
-    auto job = Controller::instance().activeConnection()->callApi<Quotient::SetPushRuleJob>("global",
-                                                                                            PushNotificationKind::kindString(kind),
-                                                                                            keyword,
-                                                                                            actions,
-                                                                                            QLatin1String(""),
-                                                                                            QLatin1String(""),
-                                                                                            pushConditions,
-                                                                                            roomId.isEmpty() ? keyword : QLatin1String(""));
+    auto job = m_connection->callApi<Quotient::SetPushRuleJob>(QLatin1String("global"),
+                                                               PushNotificationKind::kindString(kind),
+                                                               keyword,
+                                                               actions,
+                                                               QString(),
+                                                               QString(),
+                                                               pushConditions,
+                                                               roomId.isEmpty() ? keyword : QString());
     connect(job, &Quotient::BaseJob::failure, this, [job, keyword]() {
         qWarning() << QLatin1String("Unable to set push rule for keyword %1: ").arg(keyword) << job->errorString();
     });
@@ -340,7 +337,7 @@ void PushRuleModel::removeKeyword(const QString &keyword)
     }
 
     auto kind = PushNotificationKind::kindString(m_rules[index].kind);
-    auto job = Controller::instance().activeConnection()->callApi<Quotient::DeletePushRuleJob>("global", kind, m_rules[index].id);
+    auto job = m_connection->callApi<Quotient::DeletePushRuleJob>(QStringLiteral("global"), kind, m_rules[index].id);
     connect(job, &Quotient::BaseJob::failure, this, [this, job, index]() {
         qWarning() << QLatin1String("Unable to remove push rule for keyword %1: ").arg(m_rules[index].id) << job->errorString();
     });
@@ -348,27 +345,27 @@ void PushRuleModel::removeKeyword(const QString &keyword)
 
 void PushRuleModel::setNotificationRuleEnabled(const QString &kind, const QString &ruleId, bool enabled)
 {
-    auto job = Controller::instance().activeConnection()->callApi<Quotient::IsPushRuleEnabledJob>("global", kind, ruleId);
-    connect(job, &Quotient::BaseJob::success, this, [job, kind, ruleId, enabled]() {
+    auto job = m_connection->callApi<Quotient::IsPushRuleEnabledJob>(QStringLiteral("global"), kind, ruleId);
+    connect(job, &Quotient::BaseJob::success, this, [job, kind, ruleId, enabled, this]() {
         if (job->enabled() != enabled) {
-            Controller::instance().activeConnection()->callApi<Quotient::SetPushRuleEnabledJob>("global", kind, ruleId, enabled);
+            m_connection->callApi<Quotient::SetPushRuleEnabledJob>(QStringLiteral("global"), kind, ruleId, enabled);
         }
     });
 }
 
 void PushRuleModel::setNotificationRuleActions(const QString &kind, const QString &ruleId, PushNotificationAction::Action action)
 {
-    QVector<QVariant> actions;
-    if (ruleId == ".m.rule.call") {
-        actions = actionToVariant(action, "ring");
+    QList<QVariant> actions;
+    if (ruleId == QStringLiteral(".m.rule.call")) {
+        actions = actionToVariant(action, QStringLiteral("ring"));
     } else {
         actions = actionToVariant(action);
     }
 
-    Controller::instance().activeConnection()->callApi<Quotient::SetPushRuleActionsJob>("global", kind, ruleId, actions);
+    m_connection->callApi<Quotient::SetPushRuleActionsJob>(QStringLiteral("global"), kind, ruleId, actions);
 }
 
-PushNotificationAction::Action PushRuleModel::variantToAction(const QVector<QVariant> &actions, bool enabled)
+PushNotificationAction::Action PushRuleModel::variantToAction(const QList<QVariant> &actions, bool enabled)
 {
     bool notify = false;
     bool isNoisy = false;
@@ -383,10 +380,10 @@ PushNotificationAction::Action PushRuleModel::variantToAction(const QVector<QVar
         }
 
         QJsonObject action = i.toJsonObject();
-        if (action["set_tweak"].toString() == "sound") {
+        if (action[QStringLiteral("set_tweak")].toString() == QStringLiteral("sound")) {
             isNoisy = true;
-        } else if (action["set_tweak"].toString() == "highlight") {
-            if (action["value"].toString() != "false") {
+        } else if (action[QStringLiteral("set_tweak")].toString() == QStringLiteral("highlight")) {
+            if (action[QStringLiteral("value")].toString() != QStringLiteral("false")) {
                 highlightEnabled = true;
             }
         }
@@ -411,35 +408,54 @@ PushNotificationAction::Action PushRuleModel::variantToAction(const QVector<QVar
     }
 }
 
-QVector<QVariant> PushRuleModel::actionToVariant(PushNotificationAction::Action action, const QString &sound)
+QList<QVariant> PushRuleModel::actionToVariant(PushNotificationAction::Action action, const QString &sound)
 {
     // The caller should never try to set the state to unknown.
     // It exists only as a default state to diable the settings options until the actual state is retrieved from the server.
     if (action == PushNotificationAction::Unknown) {
         Q_ASSERT(false);
-        return QVector<QVariant>();
+        return QList<QVariant>();
     }
 
-    QVector<QVariant> actions;
+    QList<QVariant> actions;
 
     if (action != PushNotificationAction::Off) {
-        actions.append("notify");
+        actions.append(QStringLiteral("notify"));
     } else {
-        actions.append("dont_notify");
+        actions.append(QStringLiteral("dont_notify"));
     }
     if (action == PushNotificationAction::Noisy || action == PushNotificationAction::NoisyHighlight) {
         QJsonObject soundTweak;
-        soundTweak.insert("set_tweak", "sound");
-        soundTweak.insert("value", sound);
+        soundTweak.insert(QStringLiteral("set_tweak"), QStringLiteral("sound"));
+        soundTweak.insert(QStringLiteral("value"), sound);
         actions.append(soundTweak);
     }
     if (action == PushNotificationAction::Highlight || action == PushNotificationAction::NoisyHighlight) {
         QJsonObject highlightTweak;
-        highlightTweak.insert("set_tweak", "highlight");
+        highlightTweak.insert(QStringLiteral("set_tweak"), QStringLiteral("highlight"));
         actions.append(highlightTweak);
     }
 
     return actions;
 }
 
+NeoChatConnection *PushRuleModel::connection() const
+{
+    return m_connection;
+}
+
+void PushRuleModel::setConnection(NeoChatConnection *connection)
+{
+    if (connection == m_connection) {
+        return;
+    }
+    m_connection = connection;
+    Q_EMIT connectionChanged();
+
+    if (m_connection) {
+        connect(m_connection, &Quotient::Connection::accountDataChanged, this, &PushRuleModel::updateNotificationRules);
+        updateNotificationRules(QStringLiteral("m.push_rules"));
+    }
+}
+
 #include "moc_pushrulemodel.cpp"
