@@ -36,30 +36,30 @@ RoomListModel::RoomListModel(QObject *parent)
         m_categoryVisibility[collapsedSection] = false;
     }
 
-    connect(this, &RoomListModel::notificationCountChanged, this, [this]() {
+    connect(this, &RoomListModel::highlightCountChanged, this, [this]() {
 #if QT_VERSION < QT_VERSION_CHECK(6, 6, 0)
 #ifndef Q_OS_ANDROID
         // copied from Telegram desktop
-        const auto launcherUrl = "application://org.kde.neochat.desktop";
+        const auto launcherUrl = "application://org.kde.neochat.desktop"_ls;
         // Gnome requires that count is a 64bit integer
-        const qint64 counterSlice = std::min(m_notificationCount, 9999);
+        const qint64 counterSlice = std::min(m_highlightCount, 9999);
         QVariantMap dbusUnityProperties;
 
         if (counterSlice > 0) {
-            dbusUnityProperties["count"] = counterSlice;
-            dbusUnityProperties["count-visible"] = true;
+            dbusUnityProperties["count"_ls] = counterSlice;
+            dbusUnityProperties["count-visible"_ls] = true;
         } else {
-            dbusUnityProperties["count-visible"] = false;
+            dbusUnityProperties["count-visible"_ls] = false;
         }
 
-        auto signal = QDBusMessage::createSignal("/com/canonical/unity/launcherentry/neochat", "com.canonical.Unity.LauncherEntry", "Update");
+        auto signal = QDBusMessage::createSignal("/com/canonical/unity/launcherentry/neochat"_ls, "com.canonical.Unity.LauncherEntry"_ls, "Update"_ls);
 
         signal.setArguments({launcherUrl, dbusUnityProperties});
 
         QDBusConnection::sessionBus().send(signal);
 #endif // Q_OS_ANDROID
 #else
-        qGuiApp->setBadgeNumber(m_notificationCount);
+        qGuiApp->setBadgeNumber(m_highlightCount);
 #endif // QT_VERSION_CHECK(6, 6, 0)
     });
     connect(&SpaceHierarchyCache::instance(), &SpaceHierarchyCache::spaceHierarchyChanged, this, [this]() {
@@ -144,7 +144,6 @@ void RoomListModel::doAddRoom(Room *r)
         m_rooms.append(room);
         connectRoomSignals(room);
         Q_EMIT roomAdded(room);
-        Q_EMIT Controller::instance().roomAdded(room);
     } else {
         qCritical() << "Attempt to add nullptr to the room list";
         Q_ASSERT(false);
@@ -162,6 +161,9 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
     connect(room, &Room::notificationCountChanged, this, [this, room] {
         refresh(room);
     });
+    connect(room, &Room::highlightCountChanged, this, [this, room] {
+        refresh(room);
+    });
     connect(room, &Room::avatarChanged, this, [this, room] {
         refresh(room, {AvatarRole});
     });
@@ -178,6 +180,7 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
         refresh(room, {LastEventRole, SubtitleTextRole});
     });
     connect(room, &Room::unreadStatsChanged, this, &RoomListModel::refreshNotificationCount);
+    connect(room, &Room::highlightCountChanged, this, &RoomListModel::refreshHighlightCount);
 }
 
 int RoomListModel::notificationCount() const
@@ -185,6 +188,11 @@ int RoomListModel::notificationCount() const
     return m_notificationCount;
 }
 
+int RoomListModel::highlightCount() const
+{
+    return m_highlightCount;
+}
+
 void RoomListModel::refreshNotificationCount()
 {
     int count = 0;
@@ -198,6 +206,19 @@ void RoomListModel::refreshNotificationCount()
     Q_EMIT notificationCountChanged();
 }
 
+void RoomListModel::refreshHighlightCount()
+{
+    int count = 0;
+    for (auto room : std::as_const(m_rooms)) {
+        count += room->highlightCount();
+    }
+    if (m_highlightCount == count) {
+        return;
+    }
+    m_highlightCount = count;
+    Q_EMIT highlightCountChanged();
+}
+
 void RoomListModel::updateRoom(Room *room, Room *prev)
 {
     // There are two cases when this method is called:
@@ -297,9 +318,9 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
             return NeoChatRoomType::Normal;
         }
         QJsonObject contentJson = creationEvent->contentJson();
-        QJsonObject::const_iterator typeIter = contentJson.find("type");
+        QJsonObject::const_iterator typeIter = contentJson.find("type"_ls);
         if (typeIter != contentJson.end()) {
-            if (typeIter.value().toString() == "m.space") {
+            if (typeIter.value().toString() == "m.space"_ls) {
                 return NeoChatRoomType::Space;
             }
         }
@@ -347,11 +368,14 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     if (role == IsChildSpaceRole) {
         return SpaceHierarchyCache::instance().isChildSpace(room->id());
     }
+    if (role == ReplacementIdRole) {
+        return room->successorId();
+    }
 
     return QVariant();
 }
 
-void RoomListModel::refresh(NeoChatRoom *room, const QVector<int> &roles)
+void RoomListModel::refresh(NeoChatRoom *room, const QList<int> &roles)
 {
     const auto it = std::find(m_rooms.begin(), m_rooms.end(), room);
     if (it == m_rooms.end()) {
@@ -400,7 +424,7 @@ QString RoomListModel::categoryName(int category)
     case NeoChatRoomType::Space:
         return i18n("Spaces");
     default:
-        return "Deadbeef";
+        return {};
     }
 }
 
