@@ -11,14 +11,15 @@
 
 #include <KConfig>
 #include <KConfigGroup>
+#include <KSharedConfig>
 
 ServerListModel::ServerListModel(QObject *parent)
     : QAbstractListModel(parent)
 {
-    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup serverGroup(&dataResource, "Servers");
+    const auto stateConfig = KSharedConfig::openStateConfig();
+    const KConfigGroup serverGroup = stateConfig->group(QStringLiteral("Servers"));
 
-    QString domain = Controller::instance().activeConnection()->domain();
+    QString domain = m_connection->domain();
 
     // Add the user's homeserver
     m_servers.append(Server{
@@ -91,15 +92,15 @@ int ServerListModel::rowCount(const QModelIndex &parent) const
 
 void ServerListModel::checkServer(const QString &url)
 {
-    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup serverGroup(&dataResource, "Servers");
+    const auto stateConfig = KSharedConfig::openStateConfig();
+    const KConfigGroup serverGroup = stateConfig->group(QStringLiteral("Servers"));
 
     if (!serverGroup.hasKey(url)) {
         if (Quotient::isJobPending(m_checkServerJob)) {
             m_checkServerJob->abandon();
         }
 
-        m_checkServerJob = Controller::instance().activeConnection()->callApi<Quotient::QueryPublicRoomsJob>(url, 1);
+        m_checkServerJob = m_connection->callApi<Quotient::QueryPublicRoomsJob>(url, 1);
         connect(m_checkServerJob, &Quotient::BaseJob::success, this, [this, url] {
             Q_EMIT serverCheckComplete(url, true);
         });
@@ -108,8 +109,8 @@ void ServerListModel::checkServer(const QString &url)
 
 void ServerListModel::addServer(const QString &url)
 {
-    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup serverGroup(&dataResource, "Servers");
+    const auto stateConfig = KSharedConfig::openStateConfig();
+    KConfigGroup serverGroup = stateConfig->group(QStringLiteral("Servers"));
 
     if (!serverGroup.hasKey(url)) {
         Server newServer = Server{
@@ -125,17 +126,21 @@ void ServerListModel::addServer(const QString &url)
     }
 
     serverGroup.writeEntry(url, url);
+    stateConfig->sync();
 }
 
 void ServerListModel::removeServerAtIndex(int row)
 {
-    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
-    KConfigGroup serverGroup(&dataResource, "Servers");
+    const auto stateConfig = KSharedConfig::openStateConfig();
+    KConfigGroup serverGroup = stateConfig->group(QStringLiteral("Servers"));
+
     serverGroup.deleteEntry(data(index(row), UrlRole).toString());
 
     beginRemoveRows(QModelIndex(), row, row);
     m_servers.removeAt(row);
     endRemoveRows();
+
+    stateConfig->sync();
 }
 
 QHash<int, QByteArray> ServerListModel::roleNames() const
@@ -148,4 +153,18 @@ QHash<int, QByteArray> ServerListModel::roleNames() const
     };
 }
 
+NeoChatConnection *ServerListModel::connection() const
+{
+    return m_connection;
+}
+
+void ServerListModel::setConnection(NeoChatConnection *connection)
+{
+    if (m_connection == connection) {
+        return;
+    }
+    m_connection = connection;
+    Q_EMIT connectionChanged();
+}
+
 #include "moc_serverlistmodel.cpp"
