@@ -3,6 +3,8 @@
 
 #include "userlistmodel.h"
 
+#include <QGuiApplication>
+
 #include <Quotient/connection.h>
 #include <Quotient/events/roompowerlevelsevent.h>
 
@@ -14,6 +16,9 @@ UserListModel::UserListModel(QObject *parent)
     : QAbstractListModel(parent)
     , m_currentRoom(nullptr)
 {
+    connect(static_cast<QGuiApplication *>(QGuiApplication::instance()), &QGuiApplication::paletteChanged, this, [this]() {
+        refreshAllUsers();
+    });
 }
 
 void UserListModel::setRoom(NeoChatRoom *room)
@@ -71,8 +76,7 @@ QVariant UserListModel::data(const QModelIndex &index, int role) const
         return user->id();
     }
     if (role == AvatarRole) {
-        auto neoChatUser = static_cast<NeoChatUser *>(user);
-        return m_currentRoom->avatarForMember(neoChatUser);
+        return m_currentRoom->avatarForMember(user);
     }
     if (role == ObjectRole) {
         return QVariant::fromValue(user);
@@ -141,7 +145,7 @@ void UserListModel::userRemoved(Quotient::User *user)
     }
 }
 
-void UserListModel::refreshUser(Quotient::User *user, const QVector<int> &roles)
+void UserListModel::refreshUser(Quotient::User *user, const QList<int> &roles)
 {
     auto pos = findUserPos(user);
     if (pos != m_users.size()) {
@@ -159,17 +163,19 @@ void UserListModel::refreshAllUsers()
     }
     m_users.clear();
 
-    m_users = m_currentRoom->users();
-    std::sort(m_users.begin(), m_users.end(), m_currentRoom->memberSorter());
+    if (m_currentRoom != nullptr) {
+        m_users = m_currentRoom->users();
+        std::sort(m_users.begin(), m_users.end(), m_currentRoom->memberSorter());
 
-    for (User *user : std::as_const(m_users)) {
-        connect(user, &User::defaultAvatarChanged, this, [this, user]() {
-            refreshUser(user, {AvatarRole});
+        for (User *user : std::as_const(m_users)) {
+            connect(user, &User::defaultAvatarChanged, this, [this, user]() {
+                refreshUser(user, {AvatarRole});
+            });
+        }
+        connect(m_currentRoom->connection(), &Connection::loggedOut, this, [this]() {
+            setRoom(nullptr);
         });
     }
-    connect(m_currentRoom->connection(), &Connection::loggedOut, this, [this]() {
-        setRoom(nullptr);
-    });
     endResetModel();
     Q_EMIT usersRefreshed();
 }
