@@ -7,16 +7,26 @@
 
 #include <QCache>
 #include <QObject>
+#include <QQmlEngine>
 #include <QTextCursor>
 
 #include <QCoroTask>
+#include <Quotient/user.h>
 
-#include "neochatuser.h"
 #include "pollhandler.h"
 
+namespace Quotient
+{
+class User;
+}
+
+class ChatBarCache;
+
 class PushNotificationState : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("")
 
 public:
     /**
@@ -32,17 +42,6 @@ public:
     Q_ENUM(State)
 };
 
-/**
- * @brief Defines a user mention in the current chat or edit text.
- */
-struct Mention {
-    QTextCursor cursor; /**< Contains the mention's text and position in the text.  */
-    QString text; /**< The inserted text of the mention. */
-    int start = 0; /**< Start position of the mention. */
-    int position = 0; /**< End position of the mention. */
-    QString id; /**< The id the mention (used to create link when sending the message). */
-};
-
 /**
  * @class NeoChatRoom
  *
@@ -58,13 +57,15 @@ struct Mention {
 class NeoChatRoom : public Quotient::Room
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("")
 
     /**
      * @brief A list of users currently typing in the room.
      *
      * The list does not include the local user.
      *
-     * This is different to getting a list of NeoChatUser objects or Quotient::User objects
+     * This is different to getting a list of Quotient::User objects
      * as neither of those can provide details like the displayName or avatarMediaId
      * without the room context as these can vary from room to room. This function
      * provides the room context and puts the result as a list of QVariantMap objects.
@@ -76,7 +77,7 @@ class NeoChatRoom : public Quotient::Room
      *  - displayName - Display name in the context of this room.
      *  - display - Name in the context of this room.
      *
-     * @sa Quotient::User, NeoChatUser
+     * @sa Quotient::User
      */
     Q_PROPERTY(QVariantList usersTyping READ getUsersTyping NOTIFY typingChanged)
 
@@ -106,11 +107,6 @@ class NeoChatRoom : public Quotient::Room
      */
     Q_PROPERTY(bool readMarkerLoaded READ readMarkerLoaded NOTIFY readMarkerLoadedChanged)
 
-    /**
-     * @brief Display name with any html special characters escaped.
-     */
-    Q_PROPERTY(QString htmlSafeDisplayName READ htmlSafeDisplayName NOTIFY displayNameChanged)
-
     /**
      * @brief The avatar image to be used for the room.
      */
@@ -119,7 +115,29 @@ class NeoChatRoom : public Quotient::Room
     /**
      * @brief Get a user object for the other person in a direct chat.
      */
-    Q_PROPERTY(NeoChatUser *directChatRemoteUser READ directChatRemoteUser CONSTANT)
+    Q_PROPERTY(Quotient::User *directChatRemoteUser READ directChatRemoteUser CONSTANT)
+
+    /**
+     * @brief The Matrix IDs of this room's parents.
+     *
+     * Empty if no parent space is set.
+     */
+    Q_PROPERTY(QList<QString> parentIds READ parentIds NOTIFY parentIdsChanged)
+
+    /**
+     * @brief The current canonical parent for the room.
+     *
+     * Empty if no canonical parent is set. The write method can only be used to
+     * set an existing parent as canonical; If you wish to add a new parent and set
+     * it as canonical use the addParent method and pass true to the canonical
+     * parameter.
+     *
+     * Setting will fail if the user doesn't have the required privileges (see
+     * canModifyParent) or if the given room ID is not a parent room.
+     *
+     * @sa canModifyParent, addParent
+     */
+    Q_PROPERTY(QString canonicalParent READ canonicalParent WRITE setCanonicalParent NOTIFY canonicalParentChanged)
 
     /**
      * @brief If the room is a space.
@@ -142,6 +160,13 @@ class NeoChatRoom : public Quotient::Room
      */
     Q_PROPERTY(QString joinRule READ joinRule WRITE setJoinRule NOTIFY joinRuleChanged)
 
+    /**
+     * @brief The space IDs that members of can join this room.
+     *
+     * Empty if the join rule is not restricted.
+     */
+    Q_PROPERTY(QList<QString> restrictedIds READ restrictedIds NOTIFY joinRuleChanged)
+
     /**
      * @brief Get the maximum room version that the server supports.
      *
@@ -181,9 +206,6 @@ class NeoChatRoom : public Quotient::Room
     /**
      * @brief Whether the local user can encrypt the room.
      *
-     * Requires libQuotient 0.7 compiled with the Quotient_E2EE_ENABLED parameter to
-     * be able to return true.
-     *
      * A local user can encrypt a room if they have permission to send the m.room.encryption
      * state event.
      *
@@ -287,94 +309,14 @@ class NeoChatRoom : public Quotient::Room
     Q_PROPERTY(int spaceParentPowerLevel READ spaceParentPowerLevel WRITE setSpaceParentPowerLevel NOTIFY spaceParentPowerLevelChanged)
 
     /**
-     * @brief The current text in the chatbox for the room.
-     *
-     * Due to problems with QTextDocument, unlike the other properties here,
-     * chatBoxText is *not* used to store the text when switching rooms.
+     * @brief The cache for the main chat bar in the room.
      */
-    Q_PROPERTY(QString chatBoxText READ chatBoxText WRITE setChatBoxText NOTIFY chatBoxTextChanged)
+    Q_PROPERTY(ChatBarCache *mainCache READ mainCache CONSTANT)
 
     /**
-     * @brief The text for any message currently being edited in the room.
+     * @brief The cache for the edit chat bar in the room.
      */
-    Q_PROPERTY(QString editText READ editText WRITE setEditText NOTIFY editTextChanged)
-
-    /**
-     * @brief The event id of a message being replied to.
-     *
-     * Will be QString() if not replying to a message.
-     */
-    Q_PROPERTY(QString chatBoxReplyId READ chatBoxReplyId WRITE setChatBoxReplyId NOTIFY chatBoxReplyIdChanged)
-
-    /**
-     * @brief The event id of a message being edited.
-     *
-     * Will be QString() if not editing to a message.
-     */
-    Q_PROPERTY(QString chatBoxEditId READ chatBoxEditId WRITE setChatBoxEditId NOTIFY chatBoxEditIdChanged)
-
-    /**
-     * @brief Get the user for the message being replied to.
-     *
-     * This is different to getting a NeoChatUser object or Quotient::User object
-     * as neither of those can provide details like the displayName or avatarMediaId
-     * without the room context as these can vary from room to room.
-     *
-     * Returns an empty user if not replying to a message.
-     *
-     * The user QVariantMap has the following properties:
-     *  - isLocalUser - Whether the user is the local user.
-     *  - id - The matrix ID of the user.
-     *  - displayName - Display name in the context of this room.
-     *  - avatarSource - The mxc URL for the user's avatar in the current room.
-     *  - avatarMediaId - Avatar id in the context of this room.
-     *  - color - Color for the user.
-     *  - object - The NeoChatUser object for the user.
-     *
-     * @sa getUser, Quotient::User, NeoChatUser
-     */
-    Q_PROPERTY(QVariantMap chatBoxReplyUser READ chatBoxReplyUser NOTIFY chatBoxReplyIdChanged)
-
-    /**
-     * @brief The content of the message being replied to.
-     *
-     * Will be QString() if not replying to a message.
-     */
-    Q_PROPERTY(QString chatBoxReplyMessage READ chatBoxReplyMessage NOTIFY chatBoxReplyIdChanged)
-
-    /**
-     * @brief Get the user for the message being edited.
-     *
-     * This is different to getting a NeoChatUser object or Quotient::User object
-     * as neither of those can provide details like the displayName or avatarMediaId
-     * without the room context as these can vary from room to room.
-     *
-     * Returns an empty user if not replying to a message.
-     *
-     * The user QVariantMap has the following properties:
-     *  - isLocalUser - Whether the user is the local user.
-     *  - id - The matrix ID of the user.
-     *  - displayName - Display name in the context of this room.
-     *  - avatarSource - The mxc URL for the user's avatar in the current room.
-     *  - avatarMediaId - Avatar id in the context of this room.
-     *  - color - Color for the user.
-     *  - object - The NeoChatUser object for the user.
-     *
-     * @sa getUser, Quotient::User, NeoChatUser
-     */
-    Q_PROPERTY(QVariantMap chatBoxEditUser READ chatBoxEditUser NOTIFY chatBoxEditIdChanged)
-
-    /**
-     * @brief The content of the message being edited.
-     *
-     * Will be QString() if not editing a message.
-     */
-    Q_PROPERTY(QString chatBoxEditMessage READ chatBoxEditMessage NOTIFY chatBoxEditIdChanged)
-
-    /**
-     * @brief The file path of the attachment to be sent.
-     */
-    Q_PROPERTY(QString chatBoxAttachmentPath READ chatBoxAttachmentPath WRITE setChatBoxAttachmentPath NOTIFY chatBoxAttachmentPathChanged)
+    Q_PROPERTY(ChatBarCache *editCache READ editCache CONSTANT)
 
 public:
     /**
@@ -389,32 +331,10 @@ public:
 
     explicit NeoChatRoom(Quotient::Connection *connection, QString roomId, Quotient::JoinState joinState = {});
 
-    /**
-     * @brief Get a list of users in the context of this room.
-     *
-     * This is different to getting a list of NeoChatUser objects or Quotient::User objects
-     * as neither of those can provide details like the displayName or avatarMediaId
-     * without the room context as these can vary from room to room. This function
-     * provides the room context and returns the result as a list of QVariantMap objects.
-     *
-     * @param keyword filters the users based on the displayname containing keyword.
-     * @param limit max number of user returned, -1 is infinite.
-     *
-     * @return a QVariantList containing a QVariantMap for each user with the following
-     * properties:
-     *  - id - User ID.
-     *  - displayName - Display name in the context of this room.
-     *  - avatarMediaId - Avatar id in the context of this room.
-     *  - color - Color for the user.
-     *
-     * @sa Quotient::User, NeoChatUser
-     */
-    Q_INVOKABLE [[nodiscard]] QVariantList getUsers(const QString &keyword, int limit = -1) const;
-
     /**
      * @brief Get a user in the context of this room.
      *
-     * This is different to getting a NeoChatUser object or Quotient::User object
+     * This is different to getting a Quotient::User object
      * as neither of those can provide details like the displayName or avatarMediaId
      * without the room context as these can vary from room to room. This function
      * provides the room context and outputs the result as QVariantMap.
@@ -431,16 +351,16 @@ public:
      *  - avatarSource - The mxc URL for the user's avatar in the current room.
      *  - avatarMediaId - Avatar id in the context of this room.
      *  - color - Color for the user.
-     *  - object - The NeoChatUser object for the user.
+     *  - object - The Quotient::User object for the user.
      *
-     * @sa Quotient::User, NeoChatUser
+     * @sa Quotient::User
      */
     Q_INVOKABLE [[nodiscard]] QVariantMap getUser(const QString &userID) const;
 
     /**
      * @brief Get a user in the context of this room.
      *
-     * This is different to getting a NeoChatUser object or Quotient::User object
+     * This is different to getting a Quotient::User object
      * as neither of those can provide details like the displayName or avatarMediaId
      * without the room context as these can vary from room to room. This function
      * provides the room context and outputs the result as QVariantMap.
@@ -457,11 +377,11 @@ public:
      *  - avatarSource - The mxc URL for the user's avatar in the current room.
      *  - avatarMediaId - Avatar id in the context of this room.
      *  - color - Color for the user.
-     *  - object - The NeoChatUser object for the user.
+     *  - object - The Quotient::User object for the user.
      *
-     * @sa Quotient::User, NeoChatUser
+     * @sa Quotient::User
      */
-    Q_INVOKABLE [[nodiscard]] QVariantMap getUser(NeoChatUser *user) const;
+    Q_INVOKABLE [[nodiscard]] QVariantMap getUser(Quotient::User *user) const;
 
     [[nodiscard]] QVariantList getUsersTyping() const;
 
@@ -478,39 +398,6 @@ public:
      */
     [[nodiscard]] const Quotient::RoomEvent *lastEvent() const;
 
-    /**
-     * @brief Output a string for the message content ready for display.
-     *
-     * The output string is dependant upon the event type and the desired output format.
-     *
-     * For most messages this is the body content of the message. For media messages
-     * This will be the caption and for state events it will be a string specific
-     * to that event with some dynamic details about the event added.
-     *
-     * E.g. For a room topic state event the text will be:
-     *      "set the topic to: <new topic text>"
-     *
-     * @param evt the event for which a string is desired.
-     * @param format the output format, usually Qt::PlainText or Qt::RichText.
-     * @param stripNewlines whether the output should have new lines in it.
-     */
-    [[nodiscard]] QString eventToString(const Quotient::RoomEvent &evt, Qt::TextFormat format = Qt::PlainText, bool stripNewlines = false) const;
-
-    /**
-     * @brief Output a generic string for the message content ready for display.
-     *
-     * The output string is dependant upon the event type.
-     *
-     * Unlike NeoChatRoom::eventToString the string is the same for all events of
-     * the same type
-     *
-     * E.g. For a message the text will be:
-     *      "sent a message"
-     *
-     * @sa eventToString()
-     */
-    [[nodiscard]] QString eventToGenericString(const Quotient::RoomEvent &evt) const;
-
     /**
      * @brief Convenient way to call eventToString on the last event.
      *
@@ -587,9 +474,23 @@ public:
      */
     Q_INVOKABLE void reportEvent(const QString &eventId, const QString &reason);
 
-    [[nodiscard]] bool readMarkerLoaded() const;
+    Q_INVOKABLE QByteArray getEventJsonSource(const QString &eventId);
 
-    QString htmlSafeDisplayName() const;
+    /**
+     * @brief Open the media for the given event in an appropriate external app.
+     *
+     * Will do nothing if the event has no media.
+     */
+    Q_INVOKABLE void openEventMediaExternally(const QString &eventId);
+
+    /**
+     * @brief Copy the media for the given event to the clipboard.
+     *
+     * Will do nothing if the event has no media.
+     */
+    Q_INVOKABLE void copyEventMedia(const QString &eventId);
+
+    [[nodiscard]] bool readMarkerLoaded() const;
 
     /**
      * @brief Get subtitle text for room
@@ -602,16 +503,97 @@ public:
 
     [[nodiscard]] QString avatarMediaId() const;
 
-    NeoChatUser *directChatRemoteUser() const;
+    Quotient::User *directChatRemoteUser() const;
+
+    /**
+     * @brief Whether this room has one or more parent spaces set.
+     */
+    Q_INVOKABLE bool hasParent() const;
+
+    QList<QString> parentIds() const;
+
+    /**
+     * @brief Get a list of parent space objects for this room.
+     *
+     * Will only return retrun spaces that are know, i.e. the user has joined and
+     * a valid NeoChatRoom is available.
+     *
+     * @param multiLevel whether the function should recursively gather all levels
+     *        of parents
+     */
+    Q_INVOKABLE QList<NeoChatRoom *> parentObjects(bool multiLevel = false) const;
+
+    QString canonicalParent() const;
+    void setCanonicalParent(const QString &parentId);
+
+    /**
+     * @brief Whether the local user has permission to set the given space as a parent.
+     *
+     * @note This follows the rules determined in the Matrix spec
+     *       https://spec.matrix.org/v1.7/client-server-api/#mspaceparent-relationships
+     */
+    Q_INVOKABLE bool canModifyParent(const QString &parentId) const;
+
+    /**
+     * @brief Add the given room as a parent.
+     *
+     * Will fail if the user doesn't have the required privileges (see
+     * canModifyParent()).
+     *
+     * @sa canModifyParent()
+     */
+    Q_INVOKABLE void addParent(const QString &parentId, bool canonical = false, bool setParentChild = false);
+
+    /**
+     * @brief Remove the given room as a parent.
+     *
+     * Will fail if the user doesn't have the required privileges (see
+     * canModifyParent()).
+     *
+     * @sa canModifyParent()
+     */
+    Q_INVOKABLE void removeParent(const QString &parentId);
 
     [[nodiscard]] bool isSpace();
 
+    /**
+     * @brief Add the given room as a child.
+     *
+     * Will fail if the user doesn't have the required privileges or this room is
+     * not a space.
+     */
+    Q_INVOKABLE void addChild(const QString &childId, bool setChildParent = false, bool canonical = false);
+
+    /**
+     * @brief Remove the given room as a child.
+     *
+     * Will fail if the user doesn't have the required privileges or this room is
+     * not a space.
+     */
+    Q_INVOKABLE void removeChild(const QString &childId, bool unsetChildParent = false);
+
     bool isInvite() const;
 
     Q_INVOKABLE void clearInvitationNotification();
 
     [[nodiscard]] QString joinRule() const;
-    void setJoinRule(const QString &joinRule);
+
+    /**
+     * @brief Set the join rule for the room.
+     *
+     * Will fail if the user doesn't have the required privileges.
+     *
+     * @param joinRule the join rule [public, knock, invite, private, restricted].
+     * @param allowedSpaces only used when the join rule is restricted. This is a
+     *        list of space Matrix IDs that members of can join without an invite.
+     *        If the rule is restricted and this list is empty it is treated as a join
+     *        rule of private instead.
+     *
+     * @sa https://spec.matrix.org/latest/client-server-api/#mroomjoin_rules
+     */
+    Q_INVOKABLE void setJoinRule(const QString &joinRule, const QList<QString> &allowedSpaces = {});
+
+    QList<QString> restrictedIds() const;
 
     int maxRoomVersion() const;
 
@@ -746,46 +728,9 @@ public:
     [[nodiscard]] int spaceParentPowerLevel() const;
     void setSpaceParentPowerLevel(const int &newPowerLevel);
 
-    QString chatBoxText() const;
-    void setChatBoxText(const QString &text);
+    ChatBarCache *mainCache() const;
 
-    QString editText() const;
-    void setEditText(const QString &text);
-
-    QString chatBoxReplyId() const;
-    void setChatBoxReplyId(const QString &replyId);
-
-    QVariantMap chatBoxReplyUser() const;
-    QString chatBoxReplyMessage() const;
-
-    QString chatBoxEditId() const;
-    void setChatBoxEditId(const QString &editId);
-
-    QVariantMap chatBoxEditUser() const;
-    QString chatBoxEditMessage() const;
-
-    QString chatBoxAttachmentPath() const;
-    void setChatBoxAttachmentPath(const QString &attachmentPath);
-
-    /**
-     * @brief Retrieve the mentions for the current chatbox text.
-     */
-    QVector<Mention> *mentions();
-
-    /**
-     * @brief Retrieve the mentions for the current edit text.
-     */
-    QVector<Mention> *editMentions();
-
-    /**
-     * @brief Get the saved chatbox text for the room.
-     */
-    QString savedText() const;
-
-    /**
-     * @brief Save the chatbox text for the room.
-     */
-    void setSavedText(const QString &savedText);
+    ChatBarCache *editCache() const;
 
     /**
      * @brief Reply to the last message sent in the timeline.
@@ -818,7 +763,7 @@ public:
      */
     Q_INVOKABLE QByteArray roomAcountDataJson(const QString &eventType);
 
-    Q_INVOKABLE [[nodiscard]] QUrl avatarForMember(NeoChatUser *user) const;
+    Q_INVOKABLE [[nodiscard]] QUrl avatarForMember(Quotient::User *user) const;
 
     /**
      * @brief Returns the event that is being replied to. This includes events that were manually loaded using NeoChatRoom::loadReply.
@@ -832,6 +777,11 @@ public:
      */
     Q_INVOKABLE void loadReply(const QString &eventId, const QString &replyId);
 
+    /**
+     * If we're invited to this room, the user that invited us. Undefined in other cases.
+     */
+    Q_INVOKABLE Quotient::User *invitingUser() const;
+
 private:
     QSet<const Quotient::RoomEvent *> highlights;
 
@@ -852,14 +802,9 @@ private:
 
     std::unique_ptr<Quotient::RoomEvent> m_cachedEvent;
 
-    QString m_chatBoxText;
-    QString m_editText;
-    QString m_chatBoxReplyId;
-    QString m_chatBoxEditId;
-    QString m_chatBoxAttachmentPath;
-    QVector<Mention> m_mentions;
-    QVector<Mention> m_editMentions;
-    QString m_savedText;
+    ChatBarCache *m_mainCache;
+    ChatBarCache *m_editCache;
+
     QCache<QString, PollHandler> m_polls;
     std::vector<Quotient::event_ptr_tt<Quotient::RoomEvent>> m_extraEvents;
 
@@ -875,16 +820,13 @@ Q_SIGNALS:
     void fileUploadingProgressChanged();
     void backgroundChanged();
     void readMarkerLoadedChanged();
+    void parentIdsChanged();
+    void canonicalParentChanged();
     void lastActiveTimeChanged();
     void isInviteChanged();
     void displayNameChanged();
     void pushNotificationStateChanged(PushNotificationState::State state);
     void showMessage(MessageType messageType, const QString &message);
-    void chatBoxTextChanged();
-    void editTextChanged();
-    void chatBoxReplyIdChanged();
-    void chatBoxEditIdChanged();
-    void chatBoxAttachmentPathChanged();
     void canEncryptRoomChanged();
     void joinRuleChanged();
     void historyVisibilityChanged();
@@ -954,7 +896,8 @@ public Q_SLOTS:
                      const QString &cleanedText,
                      Quotient::MessageEventType type = Quotient::MessageEventType::Text,
                      const QString &replyEventId = QString(),
-                     const QString &relateToEventId = QString());
+                     const QString &relateToEventId = QString(),
+                     const QString &threadRootId = QString());
 
     /**
      * @brief Send an html message to the room.
@@ -969,7 +912,8 @@ public Q_SLOTS:
                          const QString &html,
                          Quotient::MessageEventType type = Quotient::MessageEventType::Text,
                          const QString &replyEventId = QString(),
-                         const QString &relateToEventId = QString());
+                         const QString &relateToEventId = QString(),
+                         const QString &threadRootId = QString());
 
     /**
      * @brief Set the room avatar.
