@@ -8,21 +8,20 @@
 #include <QMap>
 #include <QObject>
 #include <QPointer>
+#include <QQmlEngine>
 #include <QString>
 #include <Quotient/csapi/notifications.h>
 #include <Quotient/jobs/basejob.h>
 
-namespace Quotient
-{
-class Connection;
-}
-
+class NeoChatConnection;
 class KNotification;
 class NeoChatRoom;
 
 class PushNotificationAction : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("")
 
 public:
     /**
@@ -55,9 +54,16 @@ public:
 class NotificationsManager : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
 
 public:
     static NotificationsManager &instance();
+    static NotificationsManager *create(QQmlEngine *engine, QJSEngine *)
+    {
+        engine->setObjectOwnership(&instance(), QQmlEngine::CppOwnership);
+        return &instance();
+    }
 
     /**
      * @brief Display a native notification for an message.
@@ -80,7 +86,7 @@ public:
     /**
      * @brief Handle the notifications for the given connection.
      */
-    void handleNotifications(QPointer<Quotient::Connection> connection);
+    void handleNotifications(QPointer<NeoChatConnection> connection);
 
 private:
     explicit NotificationsManager(QObject *parent = nullptr);
@@ -90,13 +96,13 @@ private:
 
     QStringList m_connActiveJob;
 
-    bool shouldPostNotification(QPointer<Quotient::Connection> connection, const QJsonValue &notification);
+    bool shouldPostNotification(QPointer<NeoChatConnection> connection, const QJsonValue &notification);
 
     QHash<QString, KNotification *> m_notifications;
     QHash<QString, QPointer<KNotification>> m_invitations;
 
 private Q_SLOTS:
-    void processNotificationJob(QPointer<Quotient::Connection> connection, Quotient::GetNotificationsJob *job, bool initialization);
+    void processNotificationJob(QPointer<NeoChatConnection> connection, Quotient::GetNotificationsJob *job, bool initialization);
 
 private:
     QPixmap createNotificationImage(const QImage &icon, NeoChatRoom *room);
