@@ -4,8 +4,9 @@
 #include "texthandler.h"
 
 #include <QDebug>
-#include <QUrl>
+#include <QGuiApplication>
 #include <QStringLiteral>
+#include <QUrl>
 
 #include <Quotient/events/roommessageevent.h>
 #include <Quotient/util.h>
@@ -13,7 +14,9 @@
 
 #include <cmark.h>
 
-#include <Kirigami/PlatformTheme>
+#include <Kirigami/Platform/PlatformTheme>
+
+#include "utils.h"
 
 static const QStringList allowedTags = {
     QStringLiteral("font"),    QStringLiteral("del"),    QStringLiteral("h1"),         QStringLiteral("h2"),     QStringLiteral("h3"),    QStringLiteral("h4"),
@@ -131,13 +134,17 @@ QString TextHandler::handleRecieveRichText(Qt::TextFormat inputFormat, const Neo
         nextTokenType();
     }
 
+    // Apply user style to blockquotes
+    // Unfortunately some attributes can be only be used on table cells, so we need to wrap the content in one.
+    outputString.replace(TextRegex::blockQuote, QStringLiteral(R"(<blockquote><table><tr><td>“\1”</td></tr></table></blockquote>)"));
+
     // If the message is an emote add the user pill to the front of the message.
     if (event != nullptr) {
         auto e = eventCast<const Quotient::RoomMessageEvent>(event);
         if (e->msgtype() == Quotient::MessageEventType::Emote) {
-            auto author = static_cast<NeoChatUser *>(room->user(e->senderId()));
+            auto author = room->user(e->senderId());
             QString emoteString = QStringLiteral("* <a href=\"https://matrix.to/#/") + e->senderId() + QStringLiteral("\" style=\"color:")
-                + author->color().name() + QStringLiteral("\">") + author->displayname(room) + QStringLiteral("</a> ");
+                + Utils::getUserColor(author->hueF()).name() + QStringLiteral("\">") + author->displayname(room) + QStringLiteral("</a> ");
             if (outputString.startsWith(QStringLiteral("<p>"))) {
                 outputString.insert(3, emoteString);
             } else {
@@ -147,10 +154,11 @@ QString TextHandler::handleRecieveRichText(Qt::TextFormat inputFormat, const Neo
     }
 
     if (auto e = eventCast<const Quotient::RoomMessageEvent>(event)) {
-        bool isEdited =
-            !e->unsignedJson().isEmpty() && e->unsignedJson().contains("m.relations") && e->unsignedJson()["m.relations"].toObject().contains("m.replace");
+        bool isEdited = !e->unsignedJson().isEmpty() && e->unsignedJson().contains(QStringLiteral("m.relations"))
+            && e->unsignedJson()[QStringLiteral("m.relations")].toObject().contains(QStringLiteral("m.replace"));
         if (isEdited) {
-            Kirigami::PlatformTheme *theme = static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true));
+            Kirigami::Platform::PlatformTheme *theme =
+                static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true));
 
             QString editTextColor;
             if (theme != nullptr) {
@@ -164,7 +172,7 @@ QString TextHandler::handleRecieveRichText(Qt::TextFormat inputFormat, const Neo
             } else if (outputString.endsWith(QStringLiteral("</pre>")) || outputString.endsWith(QStringLiteral("</blockquote>"))
                        || outputString.endsWith(QStringLiteral("</table>")) || outputString.endsWith(QStringLiteral("</ol>"))
                        || outputString.endsWith(QStringLiteral("</ul>"))) {
-                outputString.append("<p>" + editedString + "</p>");
+                outputString.append(QStringLiteral("<p>%1</p>").arg(editedString));
             } else {
                 outputString.append(editedString);
             }
@@ -199,6 +207,9 @@ QString TextHandler::handleRecievePlainText(Qt::TextFormat inputFormat, const bo
      * arrive (e.g. in a caption body) it can then be stripped by the same code.
      */
     m_dataBuffer = markdownToHTML(m_dataBuffer);
+    // This is how \n is converted and for plain text we need it to just be <br>
+    // to prevent extra newlines being inserted.
+    m_dataBuffer.replace(QStringLiteral("<br />\n"), QStringLiteral("<br>"));
 
     if (stripNewlines) {
         m_dataBuffer.replace(QStringLiteral("<br>\n"), QStringLiteral(" "));
@@ -206,7 +217,7 @@ QString TextHandler::handleRecievePlainText(Qt::TextFormat inputFormat, const bo
         m_dataBuffer.replace(QStringLiteral("<br />\n"), QStringLiteral(" "));
         m_dataBuffer.replace(QStringLiteral("<br />"), QStringLiteral(" "));
         m_dataBuffer.replace(u'\n', QStringLiteral(" "));
-        m_dataBuffer.replace(u'\u2028', " ");
+        m_dataBuffer.replace(u'\u2028', QStringLiteral(" "));
     }
 
     // Strip all tags/attributes except code blocks which will be escaped.
@@ -219,7 +230,11 @@ QString TextHandler::handleRecievePlainText(Qt::TextFormat inputFormat, const bo
         if (m_nextTokenType == Type::TextCode) {
             nextTokenBuffer = unescapeHtml(nextTokenBuffer);
         } else if (m_nextTokenType == Type::Tag) {
-            nextTokenBuffer = QString();
+            if (getTagType() == QStringLiteral("br") && !stripNewlines) {
+                nextTokenBuffer = u'\n';
+            } else {
+                nextTokenBuffer = QString();
+            }
         }
 
         outputString.append(nextTokenBuffer);
@@ -323,7 +338,7 @@ bool TextHandler::isAllowedLink(const QString &link, bool isImg)
     const QUrl linkUrl = QUrl(link);
 
     if (isImg) {
-        return !linkUrl.isRelative() && linkUrl.scheme() == "mxc";
+        return !linkUrl.isRelative() && linkUrl.scheme() == QStringLiteral("mxc");
     } else {
         return !linkUrl.isRelative() && allowedLinkSchemes.contains(linkUrl.scheme());
     }
@@ -418,9 +433,54 @@ QString TextHandler::unescapeHtml(QString stringIn)
 
 QString TextHandler::linkifyUrls(QString stringIn)
 {
-    stringIn = stringIn.replace(TextRegex::mxId, QStringLiteral(R"(\1<a href="https://matrix.to/#/\2">\2</a>)"));
-    stringIn.replace(TextRegex::plainUrl, QStringLiteral(R"(<a href="\1">\1</a>)"));
-    stringIn = stringIn.replace(TextRegex::emailAddress, QStringLiteral(R"(<a href="mailto:\2">\1\2</a>)"));
+    QRegularExpressionMatch match;
+    int start = 0;
+    for (int index = 0; index != -1; index = stringIn.indexOf(TextRegex::mxId, start, &match)) {
+        int skip = 0;
+        if (match.captured(0).size() > 0) {
+            if (stringIn.left(index).count(QStringLiteral("<code>")) == stringIn.left(index).count(QStringLiteral("</code>"))) {
+                auto replacement = QStringLiteral("<a href=\"https://matrix.to/#/%1\">%1</a>").arg(match.captured(2));
+                stringIn = stringIn.replace(index, match.captured(0).size(), replacement);
+            } else {
+                skip = match.captured().length();
+            }
+        }
+        start = index + skip;
+        match = {};
+    }
+    start = 0;
+    match = {};
+    for (int index = 0; index != -1; index = stringIn.indexOf(TextRegex::plainUrl, start, &match)) {
+        int skip = 0;
+        if (match.captured(0).size() > 0) {
+            if (stringIn.left(index).count(QStringLiteral("<code>")) == stringIn.left(index).count(QStringLiteral("</code>"))) {
+                auto replacement = QStringLiteral("<a href=\"%1\">%1</a>").arg(match.captured(1));
+                stringIn = stringIn.replace(index, match.captured(0).size(), replacement);
+                skip = replacement.length();
+            } else {
+                skip = match.captured().length();
+            }
+        }
+        start = index + skip;
+        match = {};
+    }
+    start = 0;
+    match = {};
+    for (int index = 0; index != -1; index = stringIn.indexOf(TextRegex::emailAddress, start, &match)) {
+        int skip = 0;
+        if (match.captured(0).size() > 0) {
+            if (stringIn.left(index).count(QStringLiteral("<code>")) == stringIn.left(index).count(QStringLiteral("</code>"))) {
+                auto replacement = QStringLiteral("<a href=\"mailto:%1\">%1</a>").arg(match.captured(2));
+                stringIn = stringIn.replace(index, match.captured(0).size(), replacement);
+                skip = replacement.length();
+            } else {
+                skip = match.captured().length();
+            }
+        }
+        start = index + skip;
+        match = {};
+    }
+
     return stringIn;
 }
 
