@@ -10,18 +10,14 @@ TrayIcon::TrayIcon(QObject *parent)
     : KStatusNotifierItem(parent)
 {
     setCategory(KStatusNotifierItem::ItemCategory::Communications);
-    setIconByName("org.kde.neochat.tray");
+    setIconByName(QStringLiteral("org.kde.neochat.tray"));
     connect(this, &KStatusNotifierItem::activateRequested, this, [this] {
         KWindowSystem::setCurrentXdgActivationToken(providedToken());
-        Q_EMIT showWindow();
+        Q_EMIT toggleWindow();
     });
 
     connect(&WindowController::instance(), &WindowController::windowChanged, this, [this] {
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
         setAssociatedWindow(WindowController::instance().window());
-#else
-        Q_UNUSED(this);
-#endif
     });
 }
 
